// Copyright (C) 2001 Dominic Letourneau

#ifndef _SLEEP_CC_
#define _SLEEP_CC_

#include "Node.h"
#include "Object.h"
#include <unistd.h>
#include <csignal>
#include "BufferedNode.h"
#include "Buffer.h"
#include "Vector.h"
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>

#ifdef _WIN32
	#include <windows.h>
#endif

using namespace std;

namespace FD {

class Sleep;

DECLARE_NODE(Sleep)
/*Node
 *
 * @name Sleep
 * @category Flow
 * @description Sleep a certain amount of time.
 *
 * @output_name VALUE
 * @output_description Always return TRUE.
 *
 * @parameter_name SECONDS
 * @parameter_description Sleep x seconds.
 * @parameter_type float
 *
END*/

bool _stop_flag = false;

void signal_handler(int s)
{
    // Ctrl-C
    if(s = SIGINT){
        _stop_flag = true;
    }
}

class Sleep : public Node {
  
private:

    int outputID;
    float m_time;
    float m_mSec;
    int pt;

public:

    Sleep(string nodeName, ParameterSet params) 
    : Node(nodeName, params) 
    {
        m_time = dereference_cast<float>(parameters.get("SECONDS"));
        m_mSec = m_time*1000;
        outputID = addOutput("VALUE");
        signal(SIGINT, signal_handler);
    }
   
    ~Sleep() {}
   
    int getMilliCount(){
        timeb tb;
        ftime(&tb);
        int nCount = tb.millitm + (tb.time & 0xfffff) * 1000;
        return nCount;
    }
  
    int getMilliSpan(int nTimeStart){
        int nSpan = getMilliCount() - nTimeStart;
        if(nSpan < 0)
            nSpan += 0x100000 * 1000;
        return nSpan;
    }
   
    ObjectRef getOutput(int output_id, int count) 
    {      
        pt = getMilliCount();
        while(1){
            if(_stop_flag){
                printf("\n");
                printf("User stop\n"); //batchflowと表記を統一
                // 自プロセスを終了
                pid_t pid=getpid();
                #ifdef _WIN32
                    HANDLE hProcess;
                    hProcess = OpenProcess(PROCESS_TERMINATE, FALSE, pid);
                    if (hProcess == NULL) {
                        CloseHandle(hProcess);
                        break;
                    }
                    if (!TerminateProcess(hProcess, (unsigned int)-1)) {
                        CloseHandle(hProcess);
                        break;
                    }
                    CloseHandle(hProcess);
                #else
                    kill(pid,SIGTERM);
                #endif

                //throw new NodeException (this, "Sleep: Usr stop.", __FILE__, __LINE__);
            }
            if(getMilliSpan(pt) > m_mSec){
                break;
            }
            #ifdef _WIN32
                _sleep(1);
            #else
                usleep(1000);
            #endif
        }
      
        if (output_id==outputID) return TrueObject;
        else throw new NodeException (this, "Sleep: Unknown output id", __FILE__, __LINE__);
    }
};

}//namespace FD

#endif
